/*###################################################################################

Version     Date              Updated By        Bug Id            Description

1.0         21/07/2010        GUR24222                       Initial Version

######################################################################################*/
package com.alu.smm.stateserver.facade;

import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;
import javax.ejb.EJB;
import javax.ejb.Stateless;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.jboss.ejb3.annotation.LocalBinding;
import org.jboss.ejb3.annotation.Pool;
import org.jboss.ejb3.annotation.RemoteBinding;
import org.jboss.ejb3.annotation.defaults.PoolDefaults;
import com.alu.smm.alarmserver.common.AlarmObject;
import com.alu.smm.alarmserver.facade.local.AlarmServerCommonLocal;
import com.alu.smm.common.entity.AccState;
import com.alu.smm.common.entity.AmiDvcState;
import com.alu.smm.common.entity.MdState;
import com.alu.smm.common.entity.PSmmPOsSmmOse;
import com.alu.smm.common.entity.PltfmSrvcTimerState;
import com.alu.smm.common.entity.PsmrState;
import com.alu.smm.common.entity.UtlCtrtState;
import com.alu.smm.common.exception.SMMException;
import com.alu.smm.common.exception.constant.SMMErrorConstants;
import com.alu.smm.common.helper.SessionHelper;
import com.alu.smm.common.logger.SMMLogger;
import com.alu.smm.common.rt.core.Invoker;
import com.alu.smm.common.rt.core.RT;
import com.alu.smm.common.util.SMMServiceConstant;
import com.alu.smm.smmmessage.SMMMessage;
import com.alu.smm.smmmessage.TransactionConfig;
import com.alu.smm.stateserver.dao.StateServerDAO;
import com.alu.smm.stateserver.dao.StateServerDAOFactory;
import com.alu.smm.stateserver.exception.StateServerException;
import com.alu.smm.stateserver.facade.local.EntityStateTimeout;
import com.alu.smm.stateserver.facade.remote.EntityStateTimeoutRemote;

@Stateless
@LocalBinding(jndiBinding = "com.alu.smm.stateserver.facade.local.EntityStateTimeout")
@RemoteBinding(jndiBinding = "com.alu.smm.stateserver.facade.remote.EntityStateTimeoutRemote")
@Pool(value = PoolDefaults.POOL_IMPLEMENTATION_STRICTMAX, maxSize = RT.POOL_SIZE, timeout = 1000)
public class EntityStateTimeoutBean extends Invoker implements EntityStateTimeout, EntityStateTimeoutRemote {

  private static final SMMLogger LOGGER = SMMLogger.getSMMLogger(EntityStateTimeoutBean.class);

  @EJB
  private AlarmServerCommonLocal alarmserver;

  @Override
  public void execute(SMMMessage message) throws SMMException {
    LOGGER.debug("Inside EntityStateTimeoutBean.execute");
    Transaction transaction = null;
    TransactionConfig txconfig = null;
    Integer txnPerRun = null;
    int transCounter = 0;
    String sSmmSrvcNdId = message.getSMMServiceNodeId();
    txconfig = message.getTransactionData();
    txnPerRun = txconfig.getMaxTransPerRun();
    if (sSmmSrvcNdId == null) {
      LOGGER.info("Entity Statetimeout value for smmserviceNodeId in SMMMessage.");
      return;
    }
    else if (txnPerRun == null) {
      LOGGER.info("Entity Statetimeout value for maxTransaction perrun in SMMMessage.");
      return;
    }
    Session appSession = SessionHelper.getSession(message.getSMMProjectName(), SessionHelper.APPLICATION_SCHEMA);
    StateServerDAO stateServerDao = StateServerDAOFactory.getDAOImpl(StateServerDAOFactory.STATE_SERVER_DAO_IMPL);
    /**
     * Get the list of parameters from SMM project operational state to SMM operational state entity 
     * where maximum operation duration is set.
     */
    List<PSmmPOsSmmOse> smmOprStItemList = stateServerDao.getSmmOperationStateEntityDetails(message
            .getSMMProjectVersionId(), appSession, "EntityState");
    LOGGER.info("smmOprStItemList size is : " + smmOprStItemList.size());
    if (smmOprStItemList.size() > 0) {
      
      Date dt=new Date();
      AccState objAccState = null;
      PsmrState objPsmrState = null;
      MdState objMdState = null;
      AmiDvcState objAmiDvcState = null;
      UtlCtrtState objUtlCtrtState = null;
      Date stateChngDt = null;
      long dtDiff = 0;
      Map map = stateServerDao.getSpecificEntityDetails(smmOprStItemList, appSession, message.getSMMProjectVersionId());
      Set<PSmmPOsSmmOse> keySet = map.keySet();
      //Find Timer
      PltfmSrvcTimerState timer = stateServerDao.searchTimer(
              SMMServiceConstant.StateTimeOut.ENTITY_STATE_TIMEOUT_TIMER, appSession);
      if (keySet != null && keySet.size() > 0) {
        try {
          transaction = appSession.beginTransaction();
          for (PSmmPOsSmmOse pSmmPOsSmmOse : keySet) {

            List<Object> objects = (List<Object>) map.get(pSmmPOsSmmOse);

            for (Object object : objects) {

              if (object instanceof AccState) {

                objAccState = (AccState) object;
                stateChngDt = objAccState.getStateChangeDateTm();
                dtDiff = getDateDiff(stateChngDt);

                if (dtDiff >= pSmmPOsSmmOse.getMaxOsDura()) {                 
                  objAccState.setSmmEsVal(pSmmPOsSmmOse.getRsltngSmmOs());
                  objAccState.setStateChangeDateTm(dt);
                  if (transCounter < txnPerRun) {
                    appSession.saveOrUpdate(objAccState);
                    transCounter++ ;
                  }
                  else {
                    break;
                  }
                }
              }
              else if (object instanceof PsmrState) {

                objPsmrState = (PsmrState) object;
                stateChngDt = objPsmrState.getStateChangeDateTm();
                dtDiff = getDateDiff(stateChngDt);

                if (dtDiff >= pSmmPOsSmmOse.getMaxOsDura()) {                  
                  objPsmrState.setSmmEsVal(pSmmPOsSmmOse.getRsltngSmmOs());
                  objPsmrState.setStateChangeDateTm(dt);
                  if (transCounter < txnPerRun) {
                    appSession.saveOrUpdate(objPsmrState);
                    transCounter++ ;
                  }
                  else {
                    break;
                  }
                }
              }
              else if (object instanceof MdState) {

                objMdState = (MdState) object;
                stateChngDt = objMdState.getStateChangeDateTm();
                dtDiff = getDateDiff(stateChngDt);

                if (dtDiff >= pSmmPOsSmmOse.getMaxOsDura()) {
                  objMdState.setSmmEsVal(pSmmPOsSmmOse.getRsltngSmmOs());
                  objMdState.setStateChangeDateTm(dt);
                  if (transCounter < txnPerRun) {
                    appSession.saveOrUpdate(objMdState);
                    transCounter++ ;
                  }
                  else {
                    break;
                  }
                }
              }
              else if (object instanceof AmiDvcState) {

                objAmiDvcState = (AmiDvcState) object;
                stateChngDt = objAmiDvcState.getStateChangeDateTm();
                dtDiff = getDateDiff(stateChngDt);

                if (dtDiff >= pSmmPOsSmmOse.getMaxOsDura()) {
                  objAmiDvcState.setSmmEsVal(pSmmPOsSmmOse.getRsltngSmmOs());
                  objAmiDvcState.setStateChangeDateTm(dt);
                  if (transCounter < txnPerRun) {
                    appSession.saveOrUpdate(objAmiDvcState);
                    transCounter++ ;
                  }
                  else {
                    break;
                  }
                }
              }
              else if (object instanceof UtlCtrtState) {

                objUtlCtrtState = (UtlCtrtState) object;
                stateChngDt = objUtlCtrtState.getStateChangeDateTm();
                dtDiff = getDateDiff(stateChngDt);

                if (dtDiff >= pSmmPOsSmmOse.getMaxOsDura()) {
                  objUtlCtrtState.setSmmEsVal(pSmmPOsSmmOse.getRsltngSmmOs());
                  objUtlCtrtState.setStateChangeDateTm(dt);
                  if (transCounter < txnPerRun) {
                    appSession.saveOrUpdate(objUtlCtrtState);
                    transCounter++ ;
                  }
                  else {
                    break;
                  }
                }
              }
            }
          }
          transaction.commit();
        }
        catch (Exception e) {
          try {
            if (transaction != null) {
              transaction.rollback();
              LOGGER.error("Transaction was rolled back due to Exception " + e.getMessage());
            }
          }
          catch (Exception ex) {
            LOGGER.error("Cannot rollback transaction. Because of the error : " + ex.getMessage());
          }
          raiseAlarm(SMMErrorConstants.TRANSACTION_FAILURE_ES, appSession, message);
        }
        finally {
          if(appSession != null){
            try{
              Transaction tr = appSession.beginTransaction();
              if(timer != null){
                appSession.delete(timer);
              }
              tr.commit();
            }
            catch (Exception e) {
              try {
                if (transaction != null) {
                  transaction.rollback();
                  LOGGER.error("Transaction was rolled back due to Exception " + e.getMessage());
                }
              }
              catch (Exception ex) {
                LOGGER.error("Cannot rollback transaction. Because of the error : " + ex.getMessage());
              }
              raiseAlarm(SMMErrorConstants.TRANSACTION_FAILURE_ES, appSession, message);
            }

          }
          closeSession(appSession);
        }

      }
      else {
        LOGGER
                .info("No application state data for entity state change after checking time difference,value of keySet value is "
                        + keySet);
        message.setServiceActionResult(SMMServiceConstant.MISSING_ENTITY_STATE_TIMEOUT_DATA);
      }
    }
    LOGGER.info("Maximmum transaction per run value is : " + txnPerRun);
    LOGGER.info("Number of updates in application state tables : " + transCounter);

    txconfig = message.getTransactionData();
    txconfig.setMaxTransPerRun(txnPerRun);
    txconfig.setRecordsUpdatedRun(transCounter);
    Calendar cal = Calendar.getInstance();
    cal.setTime(new Date());
    message.setSMMServiceResultDateTime(cal.getTime());
    message.setTransactionData(txconfig);
    message.setServiceActionResult(SMMServiceConstant.OK);
  }

  private void closeSession(Session session) {
    if (session != null) {
      session.flush();
      session.close();
    }
  }
  /**
   * Calculate the date difference between two dates   * 
   * @param Date
   * @return long   
   */
  private long getDateDiff(Date stChngDt) {
    long diffmilis = 0;
    Calendar calUpdated = Calendar.getInstance();
    calUpdated.setTime(stChngDt);
    long currentDateMilliSec = System.currentTimeMillis();
    long updateDateMilliSec = calUpdated.getTimeInMillis();
    diffmilis = (currentDateMilliSec - updateDateMilliSec);
    return diffmilis;
  }
  /**
   * Raise platform alarm incase of exception Scenario
   * 
   * @param session
   *          - {Application Session}
   * @param alrarmId,message
   * @return
   * @throws SMMException
   */
  public void raiseAlarm(int alrarmId, Session plfAppSession, SMMMessage message) throws SMMException {
    AlarmObject alrObject = new AlarmObject();
    alrObject.setAlarmId(alrarmId);
    alrObject.setImpactDescription(SMMServiceConstant.ModuleNames.STATE_SERVER);
    String sSmmSrvcNdId = message.getSMMServiceNodeId();
    alrObject.setSmmServiceNodeId(Integer.parseInt(sSmmSrvcNdId));
    try {
      alarmserver.openAlarm(alrObject, SessionHelper.PLATFORM_NAME);
    }
    catch (Exception ex) {
      LOGGER.error("Exception raised from Alarm Server while creating alarm." + ex);
      throw new StateServerException("Exception raised from Alarm Server while creating alarm.", ex);
    }
  }

  @Override
  public boolean isSameModule(String moduleName) {
    return MODULE_NAME.equalsIgnoreCase(moduleName);
  }
}