/*###################################################################################

Version     Date              Updated By        Bug Id            Description

1.0         23/07/2010        Raman Sharma                       Initial Version

######################################################################################*/

package com.alu.smm.platformserver.facade;

import java.util.Calendar;
import java.util.Date;
import javax.annotation.Resource;
import javax.ejb.EJB;
import javax.ejb.Stateless;
import javax.ejb.Timeout;
import javax.ejb.Timer;
import javax.ejb.TimerService;
import org.hibernate.Session;
import org.jboss.ejb3.annotation.LocalBinding;
import org.jboss.ejb3.annotation.Pool;
import org.jboss.ejb3.annotation.RemoteBinding;
import org.jboss.ejb3.annotation.defaults.PoolDefaults;
import com.alu.smm.alarmserver.facade.local.AlarmServerCommonLocal;
import com.alu.smm.common.exception.SMMException;
import com.alu.smm.common.exception.constant.SMMErrorConstants;
import com.alu.smm.common.helper.SessionHelper;
import com.alu.smm.common.logger.SMMLogger;
import com.alu.smm.common.rt.core.RT;
import com.alu.smm.common.service.local.IdGeneratorLocal;
import com.alu.smm.common.util.SMMServiceConstant;
import com.alu.smm.platformserver.common.StateTimeOutInformation;
import com.alu.smm.platformserver.common.StateTimeOutServiceInvoker;
import com.alu.smm.platformserver.dao.PlatformServerDAO;
import com.alu.smm.platformserver.dao.PlatformServerDAOFactory;
import com.alu.smm.platformserver.exception.PlatformServerException;
import com.alu.smm.platformserver.facade.local.MissingMeteringDataStateTimeoutLocal;
import com.alu.smm.platformserver.facade.remote.MissingMeteringDataStateTimeoutRemote;
import com.alu.smm.smmmessage.SMMMessage;
import com.alu.smm.smmmessage.TransactionConfig;

@Stateless
@LocalBinding(jndiBinding = "com.alu.smm.platformserver.facade.local.MissingMeteringDataStateTimeoutLocal")
@RemoteBinding(jndiBinding = "com.alu.smm.platformserver.facade.remote.MissingMeteringDataStateTimeoutRemote")
@Pool(value = PoolDefaults.POOL_IMPLEMENTATION_STRICTMAX, maxSize = RT.POOL_SIZE, timeout = 1000)
public class MissingMeteringDataStateTimeOut extends StateTimeOutServiceInvoker implements
        MissingMeteringDataStateTimeoutLocal, MissingMeteringDataStateTimeoutRemote {

  private static final SMMLogger LOGGER = SMMLogger.getSMMLogger(MissingMeteringDataStateTimeOut.class);

  @Resource
  private TimerService timerService;
  @EJB
  private IdGeneratorLocal idGenerator;
  @EJB
  private AlarmServerCommonLocal alarmserver;

  @Override
  @Timeout
  public void runOnTimerExpiration(Timer timer) throws PlatformServerException {
    LOGGER.info("Inside runOnTimerExpiration");
    StateTimeOutInformation timerInformation = (StateTimeOutInformation) timer.getInfo();
    Session session = null;
    PlatformServerDAO dao = PlatformServerDAOFactory.getDAOImpl(PlatformServerDAOFactory.PLATFORM_SERVER_DAO_IMPL);
    try {
      session = SessionHelper.getSession(timerInformation.getProjectName(), SessionHelper.APPLICATION_SCHEMA);
      Session pfSession = SessionHelper.getSession(SessionHelper.PLATFORM_NAME, SessionHelper.PLF_APP_SCHEMA);
      if ( !dao.isTimerAlreadyRunning(session, getTimerName())) {
        SMMMessage message = createSMMMessage(timerInformation, session,pfSession, dao);
        dao.saveTimer(TIMER_NAME, session);
        sendMessageToQueue(message);
      }
    }
    catch (PlatformServerException ex) {
      raiseAlarm(ex);
    }
    catch (SMMException e) {
      LOGGER.error("Unable to run on Timer Expiry. Error : " + e.getMessage() + " .Cause : " + e.getCause());
    }
    finally {
      closeSession(session);
    }
  }

  private SMMMessage createSMMMessage(StateTimeOutInformation timerInformation, Session prjSession,
          Session pfSession,PlatformServerDAO dao) throws PlatformServerException {
    LOGGER.debug("Inside createSMMMessage");
    SMMMessage message = new SMMMessage();
    message.setSMMProjectName(timerInformation.getProjectName());
    message.setSMMProjectServiceActionName(SMM_PROJECT_ACTION_NAME);
    message.setSMMProjectVersionId(timerInformation.getProjectVersionId());
    message.setTransactionData(new TransactionConfig());
    message.getTransactionData().setMaxTransPerRun(timerInformation.getMaxTransPerRun());
    message.setSMMprojectAPIServiceName(SMM_PROJECT_API_SERVICE_NAME);
    message.setSMMServiceName(SMM_SERVICE_NAME);
    message.setInternalSMMServiceId(String.valueOf(idGenerator.newId()));
    message.setServiceActionResult(SMMServiceConstant.OK);
    message.setInternalSMMRequestId(String.valueOf(idGenerator.newId()));
    Calendar cal = Calendar.getInstance();
    cal.setTime(new Date());
    message.setSMMRequestDateTime(cal.getTime());
    message.setSMMServiceDateTime(cal.getTime());
    message.setSMMServiceFlowId(dao.getServiceFlowId(SMM_PROJECT_ACTION_NAME, timerInformation.getProjectVersionId(),
            prjSession));
    Integer nodeId = searchNoddeId(pfSession);
    if(nodeId != null){
      message.setSMMServiceNodeId(nodeId.toString());
    }
    return message;
  }

  @Override
  public String getTimerName() {
    return TIMER_NAME;
  }

  @Override
  public TimerService getTimerService() {
    return timerService;
  }

  @Override
  public boolean checkPreConditionSpecificToAction(Integer projectVersionId, Session session)
          throws PlatformServerException {
    LOGGER.debug("Inside checkPreConditionSpecificToAction. Step 4");
    PlatformServerDAO dao = PlatformServerDAOFactory.getDAOImpl(PlatformServerDAOFactory.PLATFORM_SERVER_DAO_IMPL);
    boolean result = dao.checkStateTimeOut(session, projectVersionId,
            SMMServiceConstant.StateTimeOut.MISSING_METERING_DATA_STATE);
    result = result && dao.checkInPSmmPOsSmmOse(session, projectVersionId, "maxLackngMdDura");
    if ( !result) {
      throw new PlatformServerException(SMMErrorConstants.MISSING_MAX_LACKING_DURATION_STATE_ERROR);
    }
    return result;
  }

  @Override
  public AlarmServerCommonLocal getAlarmServerResource() {
    return alarmserver;
  }


}
